"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var tabbable_1 = tslib_1.__importDefault(require("tabbable"));
/**
 * Поиск всех фокусируемых элементов в переданном родителе
 * или на всей странице
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @return {HTMLElement[]} - Массив найденных элементов
 */
exports.getFocusableElements = function (parent) {
    if (parent === void 0) { parent = document; }
    if (!parent || !parent.children.length) {
        return [];
    }
    return tabbable_1.default(parent);
};
/**
 * Поиск первого фокусируемого элемента в переданном родителе
 * или на всей странице
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @return {HTMLElement|null} - Найденный элемент или null
 */
exports.getFirstFocusableElement = function (parent) {
    if (parent === void 0) { parent = document; }
    if (!parent || !parent.children.length) {
        return null;
    }
    return exports.getFocusableElements(parent)[0] || null;
};
/**
 * Поиск следующего фокусируемого элемента в переданном родителе
 * или на всей странице
 * @param {HTMLElement} current - Текущий фокусируемый элемент
 * @param {Element|Document|null} [parent=document] - Родительский элемент,
 * внутри которого осуществляется поиск.
 * @param {boolean} [recursive=true] - Разрешает рекурсивный подъем вверх
 * по родителям
 * @return {HTMLElement|null} - Найденный элемент или null
 *
 * В случае поиска по всей странице, можно оставить parent === document,
 * тогда результат будет максимально точным, но возможны задержки
 * при большом кол-ве фокусируемых элементов на странице.
 * Если же передать parent === current.parentElement и recursive === true,
 * то поиск осуществится максимально быстро. Но может оказаться не учтенным
 * порядок элементов с [tabindex] > 0.
 */
exports.getNextFocusableElement = function (current, parent, recursive) {
    if (parent === void 0) { parent = document; }
    if (recursive === void 0) { recursive = true; }
    if (!parent || !parent.contains(current)) {
        return null;
    }
    if (parent.lastElementChild !== current) {
        var tabbables = exports.getFocusableElements(parent);
        if (tabbables.length > 1) {
            var currentIndex = tabbables.indexOf(current);
            var lastIndex = tabbables.length - 1;
            if (currentIndex === -1) {
                return null;
            }
            if (currentIndex < lastIndex) {
                return tabbables[currentIndex + 1];
            }
        }
    }
    return recursive ? exports.getNextFocusableElement(current, parent.parentElement) : null;
};
/**
 * Поиск ближайшего фокусируемого элемента среди переданного и его родителей
 * @param {HTMLElement} current - Текущий элемент
 * @return {HTMLElement|null} - Найденный элемент или null
 *
 */
exports.getClosestFocusableElement = function (current) {
    if (!current) {
        return null;
    }
    return tabbable_1.default.isFocusable(current) ? current : exports.getClosestFocusableElement(current.parentElement);
};
exports.default = exports.getFocusableElements;
//# sourceMappingURL=getFocusableElements.js.map